/* gestion SCI en IT    C.Dupaty 01/2003 */
/*Cette bibliothque contient toutes les fonctions de gestion de l'USART
Configuration 9800,n,8,1


void initsci(void)  configuration BAUD et interruption
char getsci(void)	retourne le premier caractre reu sur SCI
void putsci(char c)	 met c sur SCI
char *getstsci(char *s, char finst)  lit une chane de caractre sur SCI se terminant par finst
									la variable finst contient le caractre attendu en  fin de chane 
int putstsci(char *s)	 met la chane s en RAM(finit par 0)
int putrstsci(rom char *s)	 met la chane s en ROM(finit par 0)

char carUSARTdispo(void)  Cette fonction retourne 1 (vrai) si un caractre est disponible dans le buffer 
de rception et 0 si non, si les deux pointeurs sont identiques, il n'y arien dans le buffer
Chaque caractre reu dclenche une interruption qui stocke ce dernier dans un tampon mmoire. 
getsci lit dans ce tampon le plus ancien caractre reu. */

#include <p18f4620.h>

#define SIZE	40	/* taille buffer */

/*	Variables globales */
unsigned char buffer[SIZE];	/* buffer de reception */
unsigned char *ptlec;		/* pointeur de lecture */
unsigned char *ptecr;		/* pointeur d'criture */

/*	Character reception d'un caractre.
 *	boucle jusqu' rception.
 */
unsigned char getsci(void)
	{
	unsigned char c;				
	while (ptlec == ptecr);
	c = *ptlec++;			
	if (ptlec >= &buffer[SIZE])	ptlec = buffer;
	return (c);
	}

/* Cette fonction retoure 1 (vrai) si un caractre est disponible dans le buffer de rception
et 0 si non, si les deux pointeurs sont identiques, il n'y arien dans le buffer*/
char carUSARTdispo(void)
{
return (ptecr-ptlec);
}

/*	emission d'un caractre
 */
void putsci(unsigned char c)
	{
	while(!TXSTAbits.TRMT);
    TXREG=c;                           /* envoie un caractre */
    while(!PIR1bits.TXIF);
	}

/* initialise les communications et interruptions*/
void initsci(void)
{
	ptecr=buffer;
	ptlec=ptecr;		   // efface le buffer de reception
	TRISCbits.TRISC7 = 1;  // RX - Set Recieve pin for tristate
	TRISCbits.TRISC6 = 0;  // TX - Set as Output

    SPBRG = 25; 	                // configure la vitesse (BAUD) 9600
    TXSTA = 0x24; 	                  
    RCSTA = 0x90;			 		// active l'USART  CREN=1 et SPEN=1

	SPBRGH= 0x06;						// SPBRG est compose de SPBRGH et SPBRG
//  SPBRG = 0x70; 	           			/* configure la vitesse (BAUD) 1200 N 8 1*/
	SPBRG = 51; 	           			/* configure la vitesse (BAUD) 9600 N 8 1*/
    TXSTA = 0b00100000; 	                  
    RCSTA = 0b10010000;			 			/* active l'USART*/
	BAUDCONbits.BRG16=0;	// SPBRGH = 0 pour 9600 (ignore SPBRGH et 1 pour 1200
	TXSTAbits.BRGH=0;		// High Speed



    PIR1bits.TXIF=0;				// IT en emission dsactive
    PIE1bits.RCIE=1;				// IT en reception active

	RCONbits.IPEN=1;				// active la priorit sur les IT
	IPR1bits.RCIP=0;				// selectionne basse priorit pour RX (vecteur en 0x18)
	INTCONbits.GIEH=1;              // autorise IT                 
    INTCONbits.GIEL=1; 				// autorise IT basse priorit
}

/* retourne une chaine de caractre*/
unsigned char *getstsci(unsigned char *s, unsigned char finst)
	{
	unsigned char *r = s;
	char c;

	while ((c = getsci()) != finst)	*s++ = c;
	*s = 0;
	return (r);
	}

/* envoie une chaine de caractre en ROM*/
int putrstsci(rom unsigned char *s)
	{
	int n;

	for (n = 0; *s; n++, s++) putsci(*s);
	return 1;
	}



/* envoie une chaine de caractre en RAM*/
int putstsci(unsigned char *s)
	{
	int n;

	for (n = 0; *s; n++, s++) putsci(*s);
	return 1;
	}


#pragma interrupt ISR
void ISR(void)				// reception d'une interruption
{unsigned char c;
	if(PIR1bits.RCIF)		/* detection IT USART en reception*/
	{
		c=RCREG;
		*ptecr++ = c;		
		if (ptecr >= &buffer[SIZE]) ptecr = buffer;
		PIR1bits.RCIF=0;
	}	
}

#pragma code vecteurITH=0x18
void fonction_pour_it_sci(void)
{
_asm goto ISR _endasm
}
#pragma code


/* exemple d'utilisation de la bibliothque
void main(void)
{
unsigned char chaine[]="Bonjour les communications sont ouvertes\n";
unsigned char maj[]="Afficheur LCD mis  jour \n";
	OpenLcd();
	initsci();
	putstsci(chaine);
    while(1)
    {									// mettre une des 2 lignes ci dessous en commentaires
 //	putsci(getsci()+1);					// emmision / rception d'un caractre
 //	putstsci(getscsci(chaine,'*'));		// emmision / rception d'une chaine
 	getstsci(chaine,'*');				
 	SetPosition(0,0);
// 	ClearDisplay();
 	putsLCD(chaine);
 	putstsci(maj);
 	}
}
*/
